/*
 * LegoUSB Linux Kernel Driver v0.3
 * 	Linux Kernel Driver for the Lego USB IR Tower (From the Mindstorms RIS 2.0 Kit)
 *
 * Copyright (c) 2001-2002 The LegoUSB DevTeam <legousb-devel@lists.sourceforge.net>
 *   - Portions Copyright (c) 2001 Bret Thaeler <bthaeler@aol.com>
 * 
 *  The LegoUSB DevTeam Is:
 * 	 - Brendan W. McAdams <rit@users.sourceforge.net>
 * 	 - Jrgen Stueber <starblue@users.sourceforge.net>
 * 	 - Andy Bianchi <abianchi@users.sourceforge.net>
 * 	 - Bret Thaeler <bthaeler@users.sourceforge.net>
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation; either version 2 of
 *	the License, or (at your option) any later version.
 *
 *
 * This drivers was based on code from the usb-skeleton USB drivers by:
 *   Greg Kroah-Hartman (greg@kroah.com) [Copyright (c) 2001 Greg Kroah-Hartman (greg@kroah.com)]
 *
 *
 * History:
 *
 * $Log: LegoUSB.c,v $
 * Revision 1.3  2003/01/22 12:04:25  sheitmann
 * lego_write() was returning a copy of a non-existant value upon success.
 * it now correctly returns the number of bytes written.
 *
 * Revision 1.2  2003/01/22 11:08:34  sheitmann
 * Fixed bugs in the GET_COPYRIGHT and GET_CREDITS ioctl
 * functions that were causing seg faults.
 * They were passing some paraemetrs by reference when
 * they should have been passing by value.
 *
 * Revision 1.1  2003/01/20 03:39:36  huntd
 * Updated kernel module compile system to use GNU autoconf.  This is hopefully
 * more robust.
 *
 * Revision 1.1.1.1  2002/08/21 00:05:35  rit
 * Initial Import.
 *
 * This includes a proper NQC 2.4 r2 patch for USB Support, and a Linux driver.
 *
 * This is almost 100% Brets work at this point, although I've done some abstraction.
 *  
 * Things to note:
 *  - There are no docs yet =)
 *  - The Makefile for the driver is shitty.
 *  - NQC may have some hardcoded bits which prevent non-USB tower from working if built like this.  I did abstract away some of this hardcoding but no guarantees
 *
 *
 * Revision 1.1.1.1  2002/08/20 20:42:22  bmcadams
 * - BWM - 
 * Initial Import.
 *
 * This includes a proper NQC 2.4 r2 patch for USB Support, and a Linux driver.
 *
 * This is almost 100% Brets work at this point, although I've done some abstraction.
 *
 * Things to note:
 * - There are no docs yet =)
 * - The Makefile for the driver is shitty.
 * - NQC may have some hardcoded bits which prevent non-USB tower from working if built like this.  I did abstract away some of this hardcoding but no guarantees
 *
 * ---------------------------------------------------------------------
 * $Id: LegoUSB.c,v 1.3 2003/01/22 12:04:25 sheitmann Exp $
 */
#include <linux/module.h>
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/poll.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/fcntl.h>
#include <linux/spinlock.h>
#include <linux/list.h>
#include <linux/smp_lock.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/usb.h>

#include <driver.h>

static int config_num = 1;
static int usb_reset = 0;

#ifdef CONFIG_USB_DEBUG
	static int debug = 1;
#else
	static int debug;
#endif

/* Use our own dbg macro */
#undef dbg
#define dbg(format, arg...) do { if (debug) printk(KERN_DEBUG __FILE__ ": " format "\n" , ## arg); } while (0)


/* Version Information */
#define DRIVER_VERSION "v0.3"
#define DRIVER_AUTHOR "The LegoUSB DevTeam <legousb-devteam@lists.sourceforge.net>"
#define DRIVER_DESC "LegoUSB IR Tower Driver <http://legousb.sourceforge.net>"

/* Module paramaters */
MODULE_PARM(debug, "i");
MODULE_PARM_DESC(debug, "Debug enabled or not");
MODULE_PARM(config_num, "i");
MODULE_PARM_DESC(config_num, "USB config to use");
MODULE_PARM(usb_reset, "i");
MODULE_PARM_DESC(usb_reset, "Resets device on driver startup");


/* Define these values to match your device */
#define USB_LEGO_VENDOR_ID	0x0694
#define USB_LEGO_PRODUCT_ID	0x0001

/* table of devices that work with this driver */
static struct usb_device_id lego_table [] = {
	{ USB_DEVICE(USB_LEGO_VENDOR_ID, USB_LEGO_PRODUCT_ID) },
	{ }					/* Terminating entry */
};

MODULE_DEVICE_TABLE (usb, lego_table);



/* Get a minor range for your devices from the usb maintainer */
#define USB_LEGO_MINOR_BASE	200	

/* we can have up to this number of device plugged in at once */
#define MAX_DEVICES		16

#define COMMAND_TIMEOUT         (2*HZ)  /* 60 second timeout for a command */
#define LEGO_CMD_COMPLETE  1
#define LEGO_CMD_FAILURE   2


/* Structure to hold all of our device specific stuff */
struct usb_lego {
	struct usb_device *	udev;			/* save off the usb device pointer */
	struct usb_interface *	interface;		/* the interface for this device */
	devfs_handle_t		devfs;			/* devfs device node */
	unsigned char		minor;			/* the starting minor number for this device */
	unsigned char		num_ports;		/* the number of ports this device has */
	char			num_int_in;		/* number of int in endpoints we have */
	char			num_int_out;		/* number of int out endpoints we have */

	unsigned char *		int_in_buffer;		/* the buffer to receive data */
	int			int_in_size;		/* the size of the receive buffer */
	int			int_in_buffered_data;
	int			int_in_buffered_offset;
	int			int_in_polling_int;
	struct urb *		read_urb;		/* the urb used to get data */
	__u8			int_in_endpointAddr;	/* the address of the int in endpoint */

	unsigned char *		int_out_buffer;	/* the buffer to send data */
	int			int_out_size;		/* the size of the send buffer */
	struct urb *		write_urb;		/* the urb used to send data */
	__u8			int_out_endpointAddr;	/* the address of the int out endpoint */
	int			int_out_polling_int;

	struct tq_struct	tqueue;			/* task queue for line discipline waking up */
	int			open_count;		/* number of times this port has been opened */
	struct semaphore	sem;			/* locks this structure */

	wait_queue_head_t       wait_command;
	__u8                    command_finished;
	struct file		*last_doing_read;
	int			read_timeout;
	wait_queue_head_t       wait_command_write;
	__u8                    command_finished_write;
	struct file		*last_doing_write;
	int			write_timeout;
};


/* the global usb devfs handle */
extern devfs_handle_t usb_devfs_handle;


/* local function prototypes */
static ssize_t lego_read	(struct file *file, char *buffer, size_t count, loff_t *ppos);
static ssize_t lego_write	(struct file *file, const char *buffer, size_t count, loff_t *ppos);
static int lego_ioctl		(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);
static int lego_open		(struct inode *inode, struct file *file);
static int lego_release		(struct inode *inode, struct file *file);
	
static void * lego_probe	(struct usb_device *dev, unsigned int ifnum, const struct usb_device_id *id);
static void lego_disconnect	(struct usb_device *dev, void *ptr);

static void lego_write_int_callback	(struct urb *urb);
static void lego_read_int_callback	(struct urb *urb);


/* array of pointers to our devices that are currently connected */
static struct usb_lego		*minor_table[MAX_DEVICES];

/* lock to protect the minor_table structure */
static DECLARE_MUTEX (minor_table_mutex);

/* file operations needed when we register this driver */
static struct file_operations lego_fops = {
	owner:		THIS_MODULE,
	read:		lego_read,
	write:		lego_write,
	ioctl:		lego_ioctl,
	open:		lego_open,
	release:	lego_release,
};      


/* usb specific object needed to register this driver with the usb subsystem */
static struct usb_driver lego_driver = {
	name:		"LegoUSB_IRTower",
	probe:		lego_probe,
	disconnect:	lego_disconnect,
	fops:		&lego_fops,
	minor:		USB_LEGO_MINOR_BASE,
	id_table:	lego_table,
};





/**
 *	usb_lego_debug_data
 */
static inline void usb_lego_debug_data (const char *function, int size, const unsigned char *data)
{
	int i;

	if (!debug)
		return;
	
	printk (KERN_DEBUG __FILE__": %s - length = %d, data = ", 
		function, size);
	for (i = 0; i < size; ++i) {
		printk ("%.2x ", data[i]);
	}
	printk ("\n");
}

#define P2_USB_RT_PORT		(USB_TYPE_CLASS | USB_RECIP_OTHER)
#define P2_USB_FEAT_RESET	4
#define P2_USB_FEAT_C_RESET	20
static int lego_reset_usbdev(struct usb_device *dev)
{
	struct usb_device		*parent = dev->parent;
	int				i;
	/* int 				result; */
	int				port = -1;

	dbg(__FUNCTION__ " enter");
	/* Find the hub port */
	for ( i = 0; i < parent->maxchild; i++) {
		if (parent->children[i] == dev) {
			port = i;
			break;
		}
	}
	if (port < 0) return -ENOENT;

	dbg(__FUNCTION__ " devnum before reset: %d", dev->devnum);
	/* Set and clear the reset */
	usb_control_msg(parent, usb_sndctrlpipe(parent, 0), 
		USB_REQ_SET_FEATURE, P2_USB_RT_PORT, P2_USB_FEAT_RESET, 
		port+1, NULL, 0, 1*HZ);
	wait_ms(700);
	usb_control_msg(parent, usb_sndctrlpipe(parent, 0), 
		USB_REQ_CLEAR_FEATURE, P2_USB_RT_PORT, P2_USB_FEAT_C_RESET, 
		port+1, NULL, 0, 1*HZ);

	dbg(__FUNCTION__ " devnum after reset: %d", dev->devnum);
#if 0
	wait_ms(700);
	/* Set the device address */
	result=usb_set_address(dev);
	if (result < 0) {
		dbg(__FUNCTION__ " Dev not accepting address, "
			"result=%d\n", result);
		clear_bit(dev->devnum, &dev->bus->devmap.devicemap);
		dev->devnum = -1;
		return result;
	}
	dbg(__FUNCTION__ " devnum at 3: %d", dev->devnum);
#endif
	wait_ms(200);

	/* Assume we're reusing the original descriptor data */
	
	dbg(__FUNCTION__ " exit");
	return 0;
}


/**
 *	lego_delete
 */
static inline void lego_delete (struct usb_lego *dev)
{
	minor_table[dev->minor] = NULL;
	if (dev->int_in_buffer != NULL)
		kfree (dev->int_in_buffer);
	if (dev->int_out_buffer != NULL)
		kfree (dev->int_out_buffer);
	if (dev->write_urb != NULL)
		usb_free_urb (dev->write_urb);
	if (dev->read_urb != NULL)
		usb_free_urb (dev->read_urb);
	kfree (dev);
}


/**
 *	lego_open
 */
static int lego_open (struct inode *inode, struct file *file)
{
	struct usb_lego *dev = NULL;
	int subminor;
	int retval = 0;
	
	dbg(__FUNCTION__);

	subminor = MINOR (inode->i_rdev) - USB_LEGO_MINOR_BASE;
	if ((subminor < 0) ||
	    (subminor >= MAX_DEVICES)) {
		return -ENODEV;
	}

	/* increment our usage count for the module */
	MOD_INC_USE_COUNT;

	/* lock our minor table and get our local data for this minor */
	down (&minor_table_mutex);
	dev = minor_table[subminor];
	if (dev == NULL) {
		up (&minor_table_mutex);
		MOD_DEC_USE_COUNT;
		return -ENODEV;
	}

	/* lock this device */
	down (&dev->sem);

	/* unlock the minor table */
	up (&minor_table_mutex);

	if (dev->open_count == 0) {
		dev->read_timeout = 0;
		dev->write_timeout = COMMAND_TIMEOUT;
	}

	/* increment our usage count for the driver */
	++dev->open_count;

	/* save our object in the file's private structure */
	file->private_data = dev;

	/* unlock this device */
	up (&dev->sem);

	return retval;
}


/**
 *	lego_release
 */
static int lego_release (struct inode *inode, struct file *file)
{
	struct usb_lego *dev;
	int retval = 0;

	dev = (struct usb_lego *)file->private_data;
	if (dev == NULL) {
		dbg (__FUNCTION__ " - object is NULL");
		return -ENODEV;
	}

	dbg(__FUNCTION__ " - minor %d", dev->minor);

	/* lock our minor table */
	down (&minor_table_mutex);

	/* lock our device */
	down (&dev->sem);

	if (dev->open_count <= 0) {
		dbg (__FUNCTION__ " - device not opened");
		retval = -ENODEV;
		goto exit_not_opened;
	}

	if (dev->udev == NULL) {
		/* the device was unplugged before the file was released */
		up (&dev->sem);
		lego_delete (dev);
		MOD_DEC_USE_COUNT;
		up (&minor_table_mutex);
		return 0;
	}

	/* decrement our usage count for the device */
	--dev->open_count;
	if (dev->open_count <= 0) {
		/* shutdown any int writes that might be going on */
		if (dev->last_doing_write == file)
			usb_unlink_urb (dev->write_urb);
		if (dev->last_doing_read == file)
			usb_unlink_urb (dev->read_urb);
		dev->open_count = 0;
	}

	/* decrement our usage count for the module */
	MOD_DEC_USE_COUNT;

exit_not_opened:
	up (&dev->sem);
	up (&minor_table_mutex);

	return retval;
}


/**
 *	lego_read
 */
static ssize_t lego_read (struct file *file, char *buffer, size_t count, loff_t *ppos)
{
	struct usb_lego *dev;
	ssize_t read_size = 0;
	int retval = 0;
	int timeout;

	dev = (struct usb_lego *)file->private_data;
	
	dbg(__FUNCTION__ " - minor %d, count = %d", dev->minor, count);

	/* lock this object */
	down (&dev->sem);

	/* verify that the device wasn't unplugged */
	if (dev->udev == NULL) {
		up (&dev->sem);
		return -ENODEV;
	}

	if (dev->int_in_buffered_data > 0) {
		int can_return = (count > dev->int_in_buffered_data) ? dev->int_in_buffered_data : count;
		usb_lego_debug_data(__FUNCTION__ , can_return, &dev->int_in_buffer[dev->int_in_buffered_offset]);
		if (copy_to_user (buffer, &dev->int_in_buffer[dev->int_in_buffered_offset],
		  can_return))
			retval = -EFAULT;
		else {
			retval = can_return;
			dev->int_in_buffered_data -= can_return;
			dev->int_in_buffered_offset += can_return;
		}
		up (&dev->sem);
		return retval;
	}

	/* see if we are already in the middle of a write */
	if (dev->read_urb->status == -EINPROGRESS) {
		dbg ( " - already writing");
		goto exit;
	}

	read_size = (count > dev->int_in_size) ? dev->int_in_size : count;

	dev->int_in_buffered_offset = 0;
	FILL_INT_URB(dev->read_urb, dev->udev,
		      usb_rcvintpipe(dev->udev, dev->int_in_endpointAddr),
		      dev->int_in_buffer, dev->int_in_size,
		      lego_read_int_callback, dev, dev->int_in_polling_int);

	dev->command_finished = 0;
        retval = usb_submit_urb (dev->read_urb);
        if (retval) {
                dbg (" - submit urb failed");
		usb_unlink_urb(dev->read_urb);
		up (&dev->sem);
		return retval;
	}

        /* wait for the command to complete */
	if (dev->read_timeout > 0) {
		timeout = dev->read_timeout;
                dbg (" - Waiting with timeout: %d", timeout);
		while (timeout && (dev->command_finished == 0)) {
			if (signal_pending(current)) {
				dbg (" - intrupted");
				usb_unlink_urb(dev->read_urb);
				up (&dev->sem);
				return -EINTR;
			}
			up (&dev->sem);
			timeout = interruptible_sleep_on_timeout (&dev->wait_command, timeout);
			down (&dev->sem);
		}
	} else {
                dbg (" - Waiting with no timeout");
		while (dev->command_finished == 0) {
			if (signal_pending(current)) {
				dbg (" - intrupted");
				usb_unlink_urb(dev->read_urb);
				up (&dev->sem);
				return -EINTR;
			}
			up (&dev->sem);
			interruptible_sleep_on (&dev->wait_command);
			down (&dev->sem);
		}
	}
	usb_unlink_urb(dev->read_urb);

        if (dev->command_finished == 0) {
                dbg ( " - command timed out.");
                retval = -ETIMEDOUT;
                goto exit;
        }

        if (dev->command_finished == LEGO_CMD_FAILURE) {
                dbg ( " - command failed.");
                retval = -EIO;
                goto exit;
        }

        if (dev->command_finished == LEGO_CMD_COMPLETE)
                dbg ( " - command completed.");

	/* if the read was successful, copy the data to userspace */
	if (!retval) {
		int can_return;

		dev->int_in_buffered_data = dev->read_urb->actual_length;

		can_return = (count > dev->int_in_buffered_data) ? dev->int_in_buffered_data : count;
		usb_lego_debug_data(__FUNCTION__ , can_return, &dev->int_in_buffer[dev->int_in_buffered_offset]);
		if (copy_to_user (buffer, &dev->int_in_buffer[dev->int_in_buffered_offset], can_return))
			retval = -EFAULT;
		else {
			dev->int_in_buffered_data -= can_return;
			dev->int_in_buffered_offset += can_return;
			retval = can_return;
		}
	}
	
	/* unlock the device */
exit:
	up (&dev->sem);
	return retval;
}


/**
 *	lego_write
 */
static ssize_t lego_write (struct file *file, const char *buffer, size_t count, loff_t *ppos)
{
	struct usb_lego *dev;
	ssize_t bytes_written = 0;
	int retval = 0;
	int timeout;

	dev = (struct usb_lego *)file->private_data;

	dbg(__FUNCTION__ " - minor %d, count = %d", dev->minor, count);

	/* lock this object */
	down (&dev->sem);

	/* verify that the device wasn't unplugged */
	if (dev->udev == NULL) {
		retval = -ENODEV;
		up (&dev->sem);

		return retval;
	}

	/* verify that we actually have some data to write */
	if (count == 0) {
		dbg(__FUNCTION__ " - write request of 0 bytes");
		goto exit;
	}

	/* see if we are already in the middle of a write */
	if (dev->write_urb->status == -EINPROGRESS) {
		dbg ( " - already writing");
		goto exit;
	}

	/* we can only write as much as 1 urb will hold */
	bytes_written = (count > dev->int_out_size) ? 
				dev->int_out_size : count;

	/* copy the data from userspace into our urb */
	if (copy_from_user(dev->write_urb->transfer_buffer, buffer, 
			   bytes_written)) {
		retval = -EFAULT;
		goto exit;
	}

	dev->last_doing_write = file;

	usb_lego_debug_data (__FUNCTION__ , bytes_written, 
			     dev->write_urb->transfer_buffer);

	/* set up our urb */
	FILL_INT_URB(dev->write_urb, dev->udev, 
		      usb_sndintpipe(dev->udev, dev->int_out_endpointAddr),
		      dev->write_urb->transfer_buffer, bytes_written,
		      lego_write_int_callback, dev, dev->int_out_polling_int);

	/* send the data out the int port */
	dev->command_finished_write = 0;
	retval = usb_submit_urb(dev->write_urb);
	if (retval) {
		err( " - failed submitting write urb, error %d",
		    retval);
		usb_unlink_urb(dev->write_urb);
		up (&dev->sem);
		return retval;
	} else {
		retval = bytes_written;
	}

        /* wait for the command to complete */
	if (dev->write_timeout > 0) {
		timeout = dev->write_timeout;
		while (timeout && (dev->command_finished_write == 0)) {
			if (signal_pending(current)) {
				dbg (" - intrupted");
				usb_unlink_urb(dev->write_urb);
				up (&dev->sem);
				return -EINTR;
			}
			up (&dev->sem);
			timeout = interruptible_sleep_on_timeout (&dev->wait_command_write, timeout);
			down (&dev->sem);
		}
	} else {
		while (dev->command_finished_write == 0) {
			if (signal_pending(current)) {
				dbg (" - intrupted");
				usb_unlink_urb(dev->write_urb);
				up (&dev->sem);
				return -EINTR;
			}
			up (&dev->sem);
			interruptible_sleep_on(&dev->wait_command_write);
			down (&dev->sem);
		}
	}
	usb_unlink_urb(dev->write_urb);

        if (dev->command_finished_write == 0) {
                dbg ( " - command timed out.");
                retval = -ETIMEDOUT;
                goto exit;
        }

        if (dev->command_finished_write == LEGO_CMD_FAILURE) {
                dbg ( " - command failed.");
                retval = -EIO;
                goto exit;
        }

        if (dev->command_finished_write == LEGO_CMD_COMPLETE)
                dbg ( " - command completed.");
	retval = bytes_written;

exit:
	/* unlock the device */
	up (&dev->sem);

	dbg (__FUNCTION__ " - returns 0x%02x",retval);
	return retval;
}


/**
 *	lego_ioctl
 */
static int lego_ioctl (struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	struct usb_lego *dev;
	int retval = 0;

	dev = (struct usb_lego *)file->private_data;

	/* lock this object */
	down (&dev->sem);

	/* verify that the device wasn't unplugged */
	if (dev->udev == NULL) {
		up (&dev->sem);
		return -ENODEV;
	}

	dbg(__FUNCTION__ " - minor %d, cmd 0x%.4x, arg %ld", 
	    dev->minor, cmd, arg);

	switch(cmd) {
		case LEGO_TOWER_SET_PARAM: {
			LTW_REQ_GET_SET_PARM_REPLY b;
          	dbg(__FUNCTION__ __FUNCTION__ " - SET_PARM  0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_SET_PARM,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg),
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0 || b.bValue != ((int) arg >> 8)) {
					err(__FUNCTION__ " - LEGO_TOWER_SET_PARAM - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bValue 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bValue);
					retval = -b.bErrCode;
				}
			}
			}
			break;
		case LEGO_TOWER_GET_PARAM: {
			LTW_REQ_GET_SET_PARM_REPLY b;
          	dbg(__FUNCTION__ " - GET_PARM  0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_PARM,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg),
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) ||
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_PARAM - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bValue 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bValue);
					retval = -b.bErrCode;
				} else {
					retval = b.bValue;
				}
			}
			}
			break;
		case LEGO_TOWER_FLUSH: {
			LTW_REQ_FLUSH_REPLY b;
          	dbg(__FUNCTION__ " - FLUSH 0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_FLUSH,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg),
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0 || b.bBufferno != ((int) arg)) {
					err(__FUNCTION__ " - LEGO_TOWER_FLUSH - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bBufferno 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bBufferno);
					retval = -b.bErrCode;
				}
			}
			}
			break;
		case LEGO_TOWER_RESET: {
			LTW_REQ_RESET_REPLY b;
          	dbg(__FUNCTION__ " - RESET");
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_RESET,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_RESET - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode);
					retval = -b.bErrCode;
				}
			}
			}
			break;
		case LEGO_TOWER_GET_STAT: {
			LTW_REQ_GET_STAT_REPLY b;
          	dbg(__FUNCTION__ " - GET_STAT");
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_STAT,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_STAT - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode);
					retval = -b.bErrCode;
				} else {
					// memcopy
					if (copy_to_user((void *) arg, &b, sizeof(b))) {
						retval = -EFAULT;
					}
				}
			}
			}
			break;
		case LEGO_TOWER_GET_POWER: {
			LTW_REQ_GET_POWER_REPLY b;
          	dbg(__FUNCTION__ " - GET_POWER");
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_POWER,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_POWER - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bPower 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bPower);
					retval = -b.bErrCode;
				} else {
					retval = b.bPower;
				}
			}
			}
			break;
		case LEGO_TOWER_GET_LED: {
			LTW_REQ_GET_SET_LED_REPLY b;
          	dbg(__FUNCTION__ " - GET_LED  0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_LED,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg),
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0 || b.bLedId != ((int) arg)) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_LED - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bLedId 0x%02x bColor 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bLedId, b.bColor);
					retval = -b.bErrCode;
				} else {
					retval = b.bColor;
				}
			}
			}
			break;
		case LEGO_TOWER_SET_LED: {
			LTW_REQ_GET_SET_LED_REPLY b;
          	dbg(__FUNCTION__ " - SET_LED  0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_SET_LED,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg),
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0 ||
				  (b.bLedId | (b.bColor << 8)) != ((int) arg)) {
					err(__FUNCTION__ " - LEGO_TOWER_SET_LED - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bLedId 0x%02x bColor 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bLedId, b.bColor);
					retval = -b.bErrCode;
				}
			}
			}
			break;
		case LEGO_TOWER_RESET_STAT: {
			LTW_REQ_RESET_STAT_REPLY b;
          	dbg(__FUNCTION__ " - RESET_STAT");
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_RESET_STAT,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_RESET_STAT - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode);
					retval = -b.bErrCode;
				}
			}
			}
			break;
		case LEGO_TOWER_GET_PARAM_IRC: {
			LTW_REQ_GET_SET_PARM_IRC_REPLY b;
          	dbg(__FUNCTION__ " - GET_PARM_IRC  0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_PARM_IRC,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg),
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_PARAM_IRC - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bValue 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bValue);
					retval = -b.bErrCode;
				} else {
					retval = b.bValue;
				}
			}
			}
			break;
		case LEGO_TOWER_SET_PARAM_IRC: {
			LTW_REQ_GET_SET_PARM_IRC_REPLY b;
          	dbg(__FUNCTION__ " - SET_PARM_IRC  0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_SET_PARM_IRC,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg),
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0 || b.bValue != ((int) arg >> 8)) {
					err(__FUNCTION__ " - LEGO_TOWER_SET_PARAM_IRC - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bValue 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bValue);
					retval = -b.bErrCode;
				}
			}
			}
			break; 
		case LEGO_TOWER_GET_TX_SPEED: {
			LTW_REQ_GET_SET_TX_SPEED_REPLY b;
          	dbg(__FUNCTION__ " - GET_TX_SPEED");
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_TX_SPEED,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_TX_SPEED - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x wTxSpeed 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.wTxSpeed);
					retval = -b.bErrCode;
				} else {
					retval = b.wTxSpeed;
				}
			}
			}
			break;
		case LEGO_TOWER_SET_TX_SPEED: {
			LTW_REQ_GET_SET_TX_SPEED_REPLY b;
          	dbg(__FUNCTION__ " - SET_TX_SPEED  0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_SET_TX_SPEED,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg),
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0 ||
				  b.wTxSpeed != ((int) arg)) {
					err(__FUNCTION__ " - LEGO_TOWER_SET_TX_SPEED - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x wTxSpeed 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.wTxSpeed);
					retval = -b.bErrCode;
				}
			}
			}
			break;
		case LEGO_TOWER_GET_RX_SPEED: {
			LTW_REQ_GET_SET_RX_SPEED_REPLY b;
          	dbg(__FUNCTION__ " - GET_RX_SPEED");
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_RX_SPEED,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_RX_SPEED - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x wRxSpeed 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.wRxSpeed);
					retval = -b.bErrCode;
				} else {
					retval = b.wRxSpeed;
				}
			}
			}
			break;
		case LEGO_TOWER_SET_RX_SPEED: {
			LTW_REQ_GET_SET_RX_SPEED_REPLY b;
          	dbg(__FUNCTION__ " - SET_RX_SPEED 0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_SET_RX_SPEED,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg),
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0 ||
				  b.wRxSpeed != ((int) arg)) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_RX_SPEED - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x wRxSpeed 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.wRxSpeed);
					retval = -b.bErrCode;
				}
			}
			}
			break;
		case LEGO_TOWER_GET_TX_STATE: {
			LTW_REQ_GET_TX_STATE_REPLY b;
          	dbg(__FUNCTION__ " - GET_TX_STATE");
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_TX_STATE,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_TX_STATE - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bTxState 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bTxState);
					retval = -b.bErrCode;
				} else {
					retval = b.bTxState;
				}
			}
			}
			break;
		case LEGO_TOWER_GET_TX_CARRIER_FREQUENCY: {
			LTW_REQ_GET_SET_TX_CARRIER_FREQUENCY_REPLY b;
          	dbg(__FUNCTION__ " - GET_TX_CARRIER_FREQUENCY");
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_TX_CARRIER_FREQUENCY,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_TX_CARRIER_FREQUENCY - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bTxFrequency 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bTxFrequency);
					retval = -b.bErrCode;
				} else {
					retval = b.bTxFrequency;
				}
			}
			}
			break;
		case LEGO_TOWER_SET_TX_CARRIER_FREQUENCY: {
			LTW_REQ_GET_SET_TX_CARRIER_FREQUENCY_REPLY b;
          	dbg(__FUNCTION__ " - SET_TX_CARRIER_FREQUENCY 0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_SET_TX_CARRIER_FREQUENCY,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg),
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0 || b.bTxFrequency != ((int) arg)) {
					err(__FUNCTION__ " - LEGO_TOWER_SET_TX_CARRIER_FREQUENCY - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bTxFrequency 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bTxFrequency);
					retval = -b.bErrCode;
				}
			}
			}
			break;
		case LEGO_TOWER_GET_TX_CARRIER_DUTY_CYCLE: {
			LTW_REQ_GET_SET_TX_CARRIER_DUTY_CYCLE_REPLY b;
          	dbg(__FUNCTION__ " - GET_TX_CARRIER_DUTY_CYCLE");
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_TX_CARRIER_DUTY_CYCLE,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				((int) arg),
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0 || b.bTxRange != ((int) arg)) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_TX_CARRIER_DUTY_CYCLE - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bTxRange 0x%02x wTxDutyCycle %4x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bTxRange, b.wTxDutyCycle);
					retval = -b.bErrCode;
				} else {
					retval = b.wTxDutyCycle;
				}
			}
			}
			break;
		case LEGO_TOWER_SET_TX_CARRIER_DUTY_CYCLE: {
			LTW_REQ_GET_SET_TX_CARRIER_DUTY_CYCLE_REPLY b;
          	dbg(__FUNCTION__ " - SET_TX_CARRIER_DUTY_CYCLE  0x%04lx", arg);
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_SET_TX_CARRIER_DUTY_CYCLE,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				((int) arg) & 0xffff,
				((int) arg) >> 16,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0 || b.bTxRange != ((int) arg) >> 16 ||
				  b.wTxDutyCycle != (((int) arg) & 0xffff)) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_TX_CARRIER_DUTY_CYCLE - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x bTxRange 0x%02x wTxDutyCycle %4x",
					  retval, b.wNoOfBytes, b.bErrCode, b.bTxRange, b.wTxDutyCycle);
					retval = -b.bErrCode;
				}
			}
			}
			break;
		case LEGO_TOWER_GET_CAPS: {
			LTW_REQ_GET_CAPS_REPLY b;
			int r;
          	dbg(__FUNCTION__ " - GET_CAPS 0x%04lx", arg);
			if (copy_from_user(&r, (void *) arg, sizeof(r))) {
				retval = -EFAULT;
				break;
			}
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_CAPS,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				r,
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_CAPS - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode);
					retval = -b.bErrCode;
				} else {
					if (copy_to_user((void *) arg, &b, sizeof(b))) {
						retval = -EFAULT;
					}
				}
			}
			}
			break;
		case LEGO_TOWER_GET_VERSION: {
			LTW_REQ_GET_VERSION_REPLY b;
          	dbg(__FUNCTION__ " - GET_VERSION");
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_VERSION,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				&b,
				sizeof(b),
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (retval != sizeof(b) || b.wNoOfBytes != sizeof(b) || 
				  b.bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_VERSION - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x",
					  retval, b.wNoOfBytes, b.bErrCode);
					retval = -b.bErrCode;
				} else {
					// memcopy
					if (copy_to_user((void *) arg, &b, sizeof(b))) {
						retval = -EFAULT;
					}
				}
			}
			}
			break;
		case LEGO_TOWER_GET_COPYRIGHT: {
			LTW_REQ_GET_COPYRIGHT_REPLY *b;
			int len;
          	dbg(__FUNCTION__ " - GET_COPYRIGHT");
			if (copy_from_user(&len, (void *) arg, sizeof(len))) {
				retval = -EFAULT;
				break;
			}
			b = (LTW_REQ_GET_COPYRIGHT_REPLY *) kmalloc (len, GFP_KERNEL);
			if (!b) {
				retval = -ENOMEM;
				break;
			}
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_COPYRIGHT,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				b,
				len,
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (b->bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_COPYRIGHT - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x",
					  retval, b->wNoOfBytes, b->bErrCode);
					retval = -b->bErrCode;
				} else {
					// memcopy
					if (copy_to_user((void *) arg, b, b->wNoOfBytes)) {
						retval = -EFAULT;
					}
				}
			}
			kfree (b);
			}
			break;
		case LEGO_TOWER_GET_CREDITS: {
			LTW_REQ_GET_CREDITS_REPLY *b;
			int len;
          	dbg(__FUNCTION__ " - GET_CREDITS");
			if (copy_from_user(&len, (void *) arg, sizeof(len))) {
				retval = -EFAULT;
				break;
			}
			b = (LTW_REQ_GET_CREDITS_REPLY *) kmalloc (len, GFP_KERNEL);
			if (!b) {
				retval = -ENOMEM;
				break;
			}
			retval = usb_control_msg (dev->udev,
				usb_rcvctrlpipe(dev->udev, 0),
				LTW_REQ_GET_CREDITS,
				USB_TYPE_VENDOR | USB_DIR_IN | USB_RECIP_DEVICE,
				0,
				0,
				b,
				len,
				COMMAND_TIMEOUT);
			if (retval >= 0) {
				if (b->bErrCode != 0) {
					err(__FUNCTION__ " - LEGO_TOWER_GET_CREDITS - Unexpected reply from tower: len %d, wNoOfBytes 0x%02x bErrCode 0x%02x",
					  retval, b->wNoOfBytes, b->bErrCode);
					retval = -b->bErrCode;
				} else {
					// memcopy
					if (copy_to_user((void *) arg, b, b->wNoOfBytes)) {
						retval = -EFAULT;
					}
				}
			}
			kfree (b);
			}
			break;
		case LEGO_TOWER_SET_READ_TIMEOUT:
          	dbg(__FUNCTION__ " - SET_READ_TIMEOUT  0x%04lx", arg);
			retval = 0;
			dev->read_timeout = (int) arg;
			break;
		case LEGO_TOWER_SET_WRITE_TIMEOUT:
          	dbg(__FUNCTION__ " - SET_WRITE_TIMEOUT  0x%04lx", arg);
			retval = 0;
			dev->write_timeout = (int) arg;
			break;
		default:
			retval = -ENOTTY;
			break;
	}

	/* fill in your device specific stuff here */
	
	/* unlock the device */
	up (&dev->sem);
	
	/* return that we did not understand this ioctl call */
	return retval;
}


/**
 *	lego_write_bulk_callback
 */
static void lego_write_int_callback (struct urb *urb)
{
	struct usb_lego *dev = (struct usb_lego *)urb->context;

	dbg(__FUNCTION__ " - minor %d - urb count: %d error cnt: %d status: %d",
	  dev->minor, urb->actual_length, urb->error_count, urb->status);


	if ((urb->status != -ENOENT) && 
	    (urb->status != -ECONNRESET)) {
		if (urb->status == 0)
			dev->command_finished_write = LEGO_CMD_COMPLETE;
		else
			dev->command_finished_write = LEGO_CMD_FAILURE;
		wake_up_interruptible(&dev->wait_command_write);
		return;
	}

	return;
}


/**
 *	lego_read_int_callback
 */
static void lego_read_int_callback (struct urb *urb)
{
	struct usb_lego *dev = (struct usb_lego *)urb->context;

	dbg(__FUNCTION__ " - minor %d - urb count: %d error cnt: %d status: %d",
	  dev->minor, urb->actual_length, urb->error_count, urb->status);
	usb_lego_debug_data(__FUNCTION__ , urb->actual_length, dev->int_in_buffer);

	if ((urb->status != -ENOENT) && 
	    (urb->status != -ECONNRESET)) {
		if (urb->status == 0)
			dev->command_finished = LEGO_CMD_COMPLETE;
		else
			dev->command_finished = LEGO_CMD_FAILURE;
		wake_up_interruptible(&dev->wait_command);
		return;
	}

	return;
}


/**
 *	lego_probe
 *
 *	Called by the usb core when a new device is connected that it thinks
 *	this driver might be interested in.
 */
static void * lego_probe(struct usb_device *udev, unsigned int ifnum, const struct usb_device_id *id)
{
	struct usb_lego *dev = NULL;
	struct usb_interface *interface;
	struct usb_interface_descriptor *iface_desc;
	struct usb_endpoint_descriptor *endpoint;
	int minor;
	int buffer_size;
	int i;
	char name[10];

	
	dbg(__FUNCTION__ " enter");

	/* See if the device offered us matches what we can accept */
	if ((udev->descriptor.idVendor != USB_LEGO_VENDOR_ID) ||
	    (udev->descriptor.idProduct != USB_LEGO_PRODUCT_ID)) {
		dbg(__FUNCTION__ " none matching device");
		return NULL;
	}

	/* Now, if selected, do a reset */
	if (usb_reset) {
		int result = lego_reset_usbdev(udev);
		if ( result < 0 ) {
			info( " lego_reset_usbdev() failed, result=%d.\n", result);
			return NULL;
		}
		usb_reset = 0;
		info( " exiting after reset");
		return NULL;
	}

	if (usb_set_configuration(udev, config_num) < 0) {
		info ( "usb_set_configuration(%d): failed", config_num);
		return NULL;
	}

	/* select a "subminor" number (part of a minor number) */
	down (&minor_table_mutex);
	for (minor = 0; minor < MAX_DEVICES; ++minor) {
		if (minor_table[minor] == NULL)
			break;
	}
	if (minor >= MAX_DEVICES) {
		info ("Too many devices plugged in, can not handle this device.");
		goto exit;
	}

	/* allocate memory for our device state and intialize it */
	dev = kmalloc (sizeof(struct usb_lego), GFP_KERNEL);
	if (dev == NULL) {
		err ("Out of memory");
		goto exit;
	}
	minor_table[minor] = dev;

	interface = &udev->actconfig->interface[ifnum];

	init_MUTEX (&dev->sem);
	dev->udev = udev;
	dev->interface = interface;
	dev->minor = minor;
	dev->open_count = 0;

	/* set up the endpoint information */
	/* check out the endpoints */
	iface_desc = &interface->altsetting[0];
	for (i = 0; i < iface_desc->bNumEndpoints; ++i) {
		endpoint = &iface_desc->endpoint[i];

		if ((endpoint->bEndpointAddress & 0x80) &&
		    ((endpoint->bmAttributes & 3) == 0x03)) {
			/* we found a int out endpoint */
			dev->read_urb = usb_alloc_urb(0);
			if (!dev->read_urb) {
				err("No free urbs available");
				goto error;
			}
			/* we found a int in endpoint */
			buffer_size = endpoint->wMaxPacketSize;
			dev->int_in_size = buffer_size;
			dev->int_in_endpointAddr = endpoint->bEndpointAddress;
			dev->int_in_polling_int = endpoint->bInterval;
			dev->int_in_buffer = kmalloc (buffer_size, GFP_KERNEL);
			dev->int_in_buffered_data = 0;
			dev->int_in_buffered_offset = 0;
			if (!dev->int_in_buffer) {
				err("Couldn't allocate int_in_buffer");
				goto error;
			}
			FILL_INT_URB(dev->read_urb, udev, 
				      usb_rcvintpipe(udev, 
						      endpoint->bEndpointAddress),
				      dev->int_out_buffer, buffer_size,
				      lego_read_int_callback, dev, dev->int_in_polling_int);
		}
		
		if (((endpoint->bEndpointAddress & 0x80) == 0x00) &&
		    ((endpoint->bmAttributes & 3) == 0x03)) {
			/* we found a int out endpoint */
			dev->write_urb = usb_alloc_urb(0);
			if (!dev->write_urb) {
				err("No free urbs available");
				goto error;
			}
			buffer_size = endpoint->wMaxPacketSize;
			dev->int_out_size = buffer_size;
			dev->int_out_endpointAddr = endpoint->bEndpointAddress;
			dev->int_out_polling_int = endpoint->bInterval;
			dev->int_out_buffer = kmalloc (buffer_size, GFP_KERNEL);
			if (!dev->int_out_buffer) {
				err("Couldn't allocate int_out_buffer");
				goto error;
			}
			FILL_INT_URB(dev->write_urb, udev, 
				      usb_sndintpipe(udev, 
						      endpoint->bEndpointAddress),
				      dev->int_out_buffer, buffer_size,
				      lego_write_int_callback, dev,
				      dev->int_out_polling_int);
		}
	}

	/* initialize the devfs node for this device and register it */
	sprintf(name, "lego%d", dev->minor);
	
	init_waitqueue_head(&dev->wait_command);
	init_waitqueue_head(&dev->wait_command_write);
	dev->last_doing_write = 0;
	dev->last_doing_read = 0;
	dev->read_timeout = 0;
	dev->write_timeout = COMMAND_TIMEOUT;

	dev->devfs = devfs_register (usb_devfs_handle, name,
				     DEVFS_FL_DEFAULT, USB_MAJOR,
				     USB_LEGO_MINOR_BASE + dev->minor,
				     S_IFCHR | S_IRUSR | S_IWUSR | 
				     S_IRGRP | S_IWGRP | S_IROTH, 
				     &lego_fops, NULL);

	/* let the user know what node this device is now attached to */
	info ("USB lego tower device now attached to USBLego%d", dev->minor);
	goto exit;
	
error:
	lego_delete (dev);
	dev = NULL;

exit:
	up (&minor_table_mutex);
	dbg(__FUNCTION__ " exit");
	return dev;
}


/**
 *	lego_disconnect
 *
 *	Called by the usb core when the device is removed from the system.
 */
static void lego_disconnect(struct usb_device *udev, void *ptr)
{
	struct usb_lego *dev;
	int minor;

	dev = (struct usb_lego *)ptr;
	
	down (&minor_table_mutex);
	down (&dev->sem);
		
	minor = dev->minor;

	/* usb_unlink_urb (dev->write_urb); */

	/* if the device is not opened, then we clean up right now */
	if (!dev->open_count) {
		lego_delete (dev);
	} else {
		dev->udev = NULL;
		up (&dev->sem);
	}

	/* remove our devfs node */
	devfs_unregister(dev->devfs);

	info("USB lego tower #%d now disconnected", minor);
	up (&minor_table_mutex);
}



/**
 *	usb_lego_init
 */
static int __init usb_lego_init(void)
{
	int result;

	/* register this driver with the USB subsystem */
	result = usb_register(&lego_driver);
	if (result < 0) {
		err("usb_register failed for the "__FILE__" driver. Error number %d",
		    result);
		return -1;
	}

	info(DRIVER_DESC " " DRIVER_VERSION);
	return 0;
}


/**
 *	usb_lego_exit
 */
static void __exit usb_lego_exit(void)
{
	/* deregister this driver with the USB subsystem */
	usb_deregister(&lego_driver);
}


module_init (usb_lego_init);
module_exit (usb_lego_exit);

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL"); 
#endif
