/****************************************************************
 * C API to the LegoUSB Tower driver.
 * User-space applications may call these routines to configure
 * a USB Tower.
 *
 * Copyright (c) 2003 Stewart Heitmann <sheitmann@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 ****************************************************************/

#ifndef _LEGOUSBAPI_H
#define _LEGOUSBAPI_H

struct legousbapi_get_stat_reply
	{
	int NoOfRxBytes;
	int OverrunErrorCount;
	int NoiseCount;
	int FramingErrorCount;
	};

struct legousbapi_get_version_reply
	{
    int MajorVersion;
	int MinorVersion;
	int BuildNo;
	};

struct legousbapi_get_caps_reply
	{
	int CommDirections;
	int CommRange;
	int CommTransmitRate;
	int CommReceiveRate;
	int CommTransmitMinFrequency;
	int CommTransmitMaxFrequency;
	int CommTransmitMinDutyCycle;
	int CommTransmitMaxDutyCycle;
  	int NoOfBytesInUartTxBuffer;
  	int NoOfBytesInUartRxBuffer;
	};
	
extern int legousbapi_debug;

char* legousbapi_strerror(int err);
int legousbapi_get_parm(int fd, int param);
int legousbapi_set_parm(int fd, int param, int value);
int legousbapi_flush(int fd, int buffno);
int legousbapi_reset(int fd);
int legousbapi_get_stat(int fd, struct legousbapi_get_stat_reply *stat);
int legousbapi_get_power(int fd);
int legousbapi_get_led(int fd, int led_id);
int legousbapi_set_led(int fd, int led_id, int led_color);
int legousbapi_reset_stat(int fd);
int legousbapi_get_parm_irc(int fd, int param);
int legousbapi_set_parm_irc(int fd, int param, int value);
int legousbapi_get_tx_speed(int fd);
int legousbapi_set_tx_speed(int fd, int tx_speed);
int legousbapi_get_rx_speed(int fd);
int legousbapi_set_rx_speed(int fd, int rx_speed);
int legousbapi_get_tx_state(int fd);
int legousbapi_get_tx_carrier_frequency(int fd);
int legousbapi_set_tx_carrier_frequency(int fd, int carrier_frequency);
int legousbapi_get_tx_carrier_duty_cycle(int fd, int tx_range);
int legousbapi_set_tx_carrier_duty_cycle(int fd, int tx_range, int tx_duty_cycle);
int legousbapi_get_caps(int fd, int mode, struct legousbapi_get_caps_reply *caps);
int legousbapi_get_version(int fd, struct legousbapi_get_version_reply *version);
int legousbapi_get_copyright(int fd, char* copyright, int n);
int legousbapi_get_credits(int fd, char* credits, int n);
int legousbapi_set_read_timeout(int fd, int timeout);
int legousbapi_set_write_timeout(int fd, int timeout);



#endif
