/*
 * DasBlinkenLichten.c 
 * 	Act as a test application by toggling the tower's red/green lights on and off.
 *  How christmasy! 
 *
 * Brendan W. McAdams <rit@jacked-in.org>
 * 
 * Copyright (c) 2001-2002 The LegoUSB DevTeam <legousb-devel@lists.sourceforge.net>
 *   - Portions Copyright (c) 2001 Bret Thaeler <bthaeler@aol.com>
 * 
 *  The LegoUSB DevTeam Is:
 * 	 - Brendan W. McAdams <rit@users.sourceforge.net>
 * 	 - Jrgen Stueber <starblue@users.sourceforge.net>
 * 	 - Andy Bianchi <abianchi@users.sourceforge.net>
 * 	 - Bret Thaeler <bthaeler@users.sourceforge.net>
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation; either version 2 of
 *	the License, or (at your option) any later version.
 *
 *
 * This program tests the driver by using IOCTL to turn the red and green lights on and off.
 * It needs alot of tuning; it's not a very well written program!
 *
 * History:
 *
 * $Log: dasblinkenlichten.c,v $
 * Revision 1.1  2002/08/22 19:00:07  rit
 * First working full version of this program!
 *
 *
 * ---------------------------------------------------------------------
 * $Id: dasblinkenlichten.c,v 1.1 2002/08/22 19:00:07 rit Exp $
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <driver.h>
#include <linux/types.h>
#define DEFAULT_TOWER_NAME "/dev/usb/lego0"

main(int argc, char *argv[]) {
	char *device_node = DEFAULT_TOWER_NAME; /* The device node to open */
	int LegoTower; /* A reference to our open device node filehandle */
	int ioret = 0; /* IOCTL return values... */
	int i = 0; /* loop iteration var */

	/* 
	 * Define 'Constants' for the various byte segments we'll send to the tower...
	 * I'm working on fixing the driver so this shit is done in the driver, or at least providing 
	 * helper functions....
	 */
	const Word HW_CONTROLLED_LED = ((LTW_ID_LED_HW_CTRL & 0xff) << 8) | LTW_PARM_ID_LED_MODE; // Firmware/Hardware controls LED State
	const Word SW_CONTROLLED_LED = ((LTW_ID_LED_SW_CTRL & 0xff) << 8) | LTW_PARM_ID_LED_MODE; // Software/User app controls LED State
	const Word RED_LIGHT_ON = ((LTW_LED_COLOR_ON & 0xff) << 8) | LTW_LED_VLL; // Turn on the Red (Visual Light Link Indicator) LED
	const Word RED_LIGHT_OFF = ((LTW_LED_COLOR_OFF & 0xff) << 8) | LTW_LED_VLL; // Turn off the Red (Visual Light Link Indicator) LED
	const Word GREEN_LIGHT_ON = ((LTW_LED_COLOR_ON & 0xff) << 8) | LTW_LED_ID; // Turn on the Green (InfraRed Light (what the Mindstorms use) Indicator) LED
	const Word GREEN_LIGHT_OFF = ((LTW_LED_COLOR_OFF & 0xff) << 8) | LTW_LED_ID; // Turn off the Green (InfraRed Light (what the Mindstorms use) Indicator) LED

	printf("Opening Lego USB Tower\n");
	LegoTower = open(device_node, O_RDWR); // Open the tower!
	if (LegoTower == -1) {
		printf("Error opening USB Tower?\n");
		exit(-1);
	} else {
		printf("Successfully opened USB Tower (%i)\n", LegoTower);
	}


	/* 
 	 * By default, the USB tower controls LED status in hardware 
	 * (e.g. if it's transmitting/ receiving it turns on the appropriate LED, etc.)
	 * We're asking the tower to allow us to control LED status at a software layer.
	 * The lights act as indicators, and I believe the VLL LED also transmits its data possibly?
	 * The Green light is the indicator used by the RCX in IR (what mindstorms uses) mode.
	 * The Red light is the indicator used in VLL (Visual Light Link - used by ?Scout?) mode.
	 */
	ioret = ioctl(LegoTower, LEGO_TOWER_SET_PARAM, SW_CONTROLLED_LED);
	printf("Beginning LED Tests.  This will take ~60-90 seconds; you should see your towers LEDs toggle on and off.\n");

	/* Loop through a light test routine 10 times */	
	for (i = 0; i < 10; i++) {
		ioret = ioctl(LegoTower, LEGO_TOWER_SET_LED, RED_LIGHT_ON); // Red (VLL) Light On
		sleep(1);
		ioret = ioctl(LegoTower, LEGO_TOWER_SET_LED, RED_LIGHT_OFF); // Red (VLL) Light Off
		sleep(1);
		ioret = ioctl(LegoTower, LEGO_TOWER_SET_LED, GREEN_LIGHT_ON); // Green (IR) Light On
		sleep(1);
		ioret = ioctl(LegoTower, LEGO_TOWER_SET_LED, GREEN_LIGHT_OFF); // Green (IR) Light Off
		sleep(1);
		ioret = ioctl(LegoTower, LEGO_TOWER_SET_LED, GREEN_LIGHT_ON); // Green (IR) Light On
		ioret = ioctl(LegoTower, LEGO_TOWER_SET_LED, RED_LIGHT_ON); // Red (VLL) Light On
		sleep(1);
		ioret = ioctl(LegoTower, LEGO_TOWER_SET_LED, GREEN_LIGHT_OFF); // Green (IR) Light Off
		ioret = ioctl(LegoTower, LEGO_TOWER_SET_LED, RED_LIGHT_OFF); // Red (VLL) Light Off
		sleep(1);
	}

	ioret = ioctl(LegoTower, LEGO_TOWER_SET_PARAM, HW_CONTROLLED_LED); // Set the LED back to Hardware mode

	printf("Testing completed.  Closing Tower connection!\n");
	close(LegoTower); // Close our connection to the tower
	return 1;
}
