/****************************************************************
 * Test utility for the LegoUSB driver.
 *
 * Copyright (c) 2003 Stewart Heitmann <sheitmann@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 ****************************************************************/

#include "argtable.h"
#include "legousbapi.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <libgen.h>

/* values extracted from command line args */
char device[100];
int help;
int param;
int value;

void parse_args(int argc, char **argv)
	{
	const char descrip[]="Sends a SET_PARM command to the Lego USB tower.";
	arg_rec argtable[] =
		{
		{ NULL,    "<param>",    arg_int, &param, NULL,           "\t\t USB tower parameter code\n"
																"\t\t\t   0x01 = MODE\n"
																"\t\t\t   0x02 = RANGE\n"
																"\t\t\t   0x03 = ERRDETECT\n"
																"\t\t\t   0x04 = ERRSTATUS\n"
																"\t\t\t   0x97 = ENDIAN\n"
																"\t\t\t   0x98 = ID_LED_MODE\n"
																"\t\t\t   0x99 = ERROR_SIGNAL"},
		{ NULL,    "<value>",    arg_int, &value, NULL,           "\t\t value to assign the specified parameter"},
		{ "-dev ", "<filename>", arg_str, device, "/dev/usb/lego0", "\t USB tower device (/dev/usb/lego0)" },
		{ "-help",  NULL,        arg_lit, &help,  "0",            "\t\t show command line options"  },
		};
	const size_t narg = sizeof(argtable)/sizeof(arg_rec);
   	char cmdline[200], errmsg[200], errmark[200];
    int argerr;

	/* parse the command line args */
   	argerr = arg_scanargv(argc,argv,argtable,narg,cmdline,errmsg,errmark);

	/* if there were no cmd line args or if -help option then... */
	if (argc==1 || help)
    	{
     	/* display program usage and exit. */
		printf("%s\n",descrip);
     	printf("usage:\n   %s %s\nwhere:\n", basename(argv[0]), arg_syntax(argtable,narg));
     	printf("%s\n",arg_glossary(argtable,narg,"   "));
     	exit(0);
     	}

	/* if there was an error in the command line then.. */
	if (argerr==0)
		{
		/* print error message and exit */
		printf("ERROR: %s\n", cmdline);
		printf("       %s %s\n", errmark, errmsg);
		exit(1);
		}
	}

int main(int argc, char **argv)
	{
	int fd,result;

	/* parse the command line args */
	parse_args(argc,argv);

 	/* open the USB tower device */
	fd = open(device, O_RDWR);
	if (fd<0)
		{
	    perror(device);
		return 1;
		}

 	/* execute the usb tower command with debug trace enabled */
	legousbapi_debug=1;
	result = legousbapi_set_parm(fd, param, value);

	/* close the usb tower */
	close(fd);

  	return 0;
  	}
