%option noyywrap
%option  always-interactive
%x command
%{
#include <string.h>
#include "scanner.h"

#ifdef TEST_SCANNER
char buffer[256];
char * messageBuffer;
int messageLength;
#else
extern char * messageBuffer;
extern int messageLength;
#endif
%}

%{
#define YY_INPUT(buf,result,max_size)                             \
{                                                                 \
   int size;                                                      \
                                                                  \
   if ( ( messageLength == 0 ) || ( messageBuffer == NULL ) )     \
     {                                                            \
        result = YY_NULL;                                         \
     }                                                            \
   else                                                           \
     {                                                            \
       if ( max_size > messageLength )                            \
         {                                                          \
           size = messageLength;                                    \
         }                                                          \
       else                                                         \
         {                                                          \
           size = max_size;                                     \
         }                                                          \
       memcpy( buf, messageBuffer, size );                       \
       if ( size <  messageLength )                                 \
         {                                                          \
           messageBuffer = messageBuffer + size;                  \
           messageLength = messageLength - size;                  \
         }                                                          \
        else                                                         \
         {                                                          \
            messageBuffer = NULL;                                  \
            messageLength = 0;                                     \
         }                                                          \
        result = size;                                               \
     }                                                          \
}
%}

DIGIT    [0-9]
NAME     [[:alpha:]][[:alnum:]_.-]*
HEXBYTE  0x[[:xdigit:]][[:xdigit:]]
NUMBER   -?[[:digit:]]+([.][[:digit:]]+)?
KICKL    [Kk]ick[ -][Ll]eft
KICKR    [Kk]ick[ -][Rr]ight
KICKS    [Kk]ick([ -][Ss]traight)?
SEP      [[:space:],.;:]+ 
WS       [[:space:]]+

%% /* Start of Rules */

<INITIAL>{NAME} {
   enum ScannerTokens ret;
#ifdef TEST_SCANNER
   printf(" got robot name %s\n", yytext );
#endif
   ret = NAME;
   BEGIN(command);
   return ret;
}

<command>{HEXBYTE} {
   enum ScannerTokens ret;
#ifdef TEST_SCANNER
   printf("got hexbyte %s\n", yytext );
#endif
   ret = HEXBYTE;
   return ret;
}

<command>{NUMBER} {
#ifdef TEST_SCANNER
  printf("got number %s\n", yytext );
#endif
  return NUMBER; 
}

<command>{KICKL} {
#ifdef TEST_SCANNER
  printf("got kick left %s\n", yytext );
#endif
  return KICKL; 
}

<command>{KICKR} {
#ifdef TEST_SCANNER
  printf("got kick right %s\n", yytext );
#endif
  return KICKR; 
}

<command>{KICKS} {
#ifdef TEST_SCANNER
  printf("got kick straight %s\n", yytext );
#endif
  return KICKS; 
}

<*>\n {
#ifdef TEST_SCANNER
  printf("got new line\n" );
#endif
  BEGIN(INITIAL);
  return NEWLINE; 
}

{SEP} {
#ifdef TEST_SCANNER
  printf("got seperator %s\n", yytext );
#endif
  return SEP; 
}

%% /* Main program */

#ifdef TEST_SCANNER
int main(int argc, char *argv[]) {
  int token;

  printf( "Type in message\n" );
  while ( fgets( buffer, sizeof( buffer ), stdin ) == buffer )
   {
      messageBuffer = buffer;
      messageLength = strlen( messageBuffer ) + 1;
      yyrestart( NULL );
      printf( "Msg(%d)=%s\n", messageLength, messageBuffer );

      while ( ( token = yylex() ) != 0 ) 
        {
          printf("Read in token %d %s\n", token, yytext );
        }
      printf( "Type in message\n" );
   }
  return 0;
}
#endif
